//Fragment Shader
#version 440

// Uniforms:
layout(location = 0) uniform mat4 u_ModelMatrix;

// SceneUBO:
//THIS NEEDS TO MATCH SceneUBO.h!
const int LIGHT_COUNT = 50;
const int SHADOWMAP_COUNT = 3;
struct Light {
    vec4 position;
	vec4 direction;
    vec4 color;
	int type;

    float constant;
    float linear;
    float quadratic;
};
layout(std140, binding = 0) uniform Scene {
	mat4 u_ViewMatrix;
	mat4 u_ProjectMatrix;

	mat4 lightViewMatrices[4];
	mat4 lightOrthoProjMatrices[4];
	vec4 cascadeFarZ;				//we use a vec4 here casue glsl wants to pad each element of a float array up to a vec4.
	vec4 u_sunDirection;
	vec4 u_sunColor;

	vec4 u_Iambient;		//ambient illumination color
	int u_LightCount;

	Light u_Lights[LIGHT_COUNT];
};

//Varying:
layout(location=0) out vec4 fragColor;
layout(location=1) in vec2 v_UV;
layout(location=2) in vec3 v_Normal;
layout(location=3) in mat3 TBN;
layout(location=6) in vec4[SHADOWMAP_COUNT] v_modelLightViewVertexPos;

//TEXTURES:
layout(binding=13) uniform sampler2DShadow shadowmap_0;
layout(binding=14) uniform sampler2DShadow shadowmap_1;
layout(binding=15) uniform sampler2DShadow shadowmap_2;

float sampleShadowPCFBlur(sampler2DShadow map, vec2 location, int radius, float bias, float depth){
	float shadow = 0.0f;
	vec2 texelSize = 1.0f / textureSize(map, 0);

	for(int x=-radius; x<=radius; x++){
		for(int y=-radius; y<=radius; y++){

			//When we sample a sampler2DShadow PCf should be done automatically. However, we need to make sure that we have the texture's compare func & mode configured properly
			float temp = texture(map, vec3(location + vec2(x,y)*texelSize, depth-bias)).r;

			//now compare the distance between the frag and sun (depth) with the value stored in the shadow map
			shadow += temp;//depth - bias > temp ? 0.1f : 1.0f;
		}
	}

	return shadow / ((2*radius+1)*(2*radius+1));
}


float getShadow(){
	float surfaceLightDot = dot(v_Normal, u_sunDirection.xyz * (-1));
	

	//glFragCoord is only between 0 and 1. This should recover the real distance to the camera
	float originalZ = gl_FragCoord.z / gl_FragCoord.w;

	//check which texture we want to sample
	int index = 4;
	for (int i=0; i<SHADOWMAP_COUNT; i++) {
		if ( abs(originalZ) < cascadeFarZ[i] ) {
			index = i;
			break;
		}
	}

	//since multiple fragments mights need to share one pixel in the shadow map it can't be 100% perfect.
	//The result is a grainy pattern on surfaces
	//Especially on surfaces that are on a steep angle from the light source this is noticeable.
	//We use the dot product to create a bias based on the angle. We use this to be more leniant on steep angles.
	float maxBias = 0.0055f;
	float minBias = 0.0012f;
	float bias = max(maxBias * (1.0f - abs(surfaceLightDot)), minBias);


	//screen coord (-1 to 1) to tex coords (0 to 1)
	vec3 texCoords = v_modelLightViewVertexPos[index].xyz * 0.5 + 0.5;

	float depth;
	if(index == 0){
		return sampleShadowPCFBlur(shadowmap_0, texCoords.xy, 1, bias, texCoords.z);
	} else if (index == 1) {
		return sampleShadowPCFBlur(shadowmap_1, texCoords.xy, 1, bias, texCoords.z);
	} else if (index == 2) {
		return sampleShadowPCFBlur(shadowmap_2, texCoords.xy, 1, bias, texCoords.z);
	}
}


void main() {
		
	fragColor = vec4(abs(v_Normal)*getShadow(), 1.0f);		//normals

}