#version 440

// Attributes (per vertex information)
layout (location = 0) in vec3 a_vertex; // <vec2 pos, vec2 tex>
layout(location = 4) in vec2 a_UV;

layout (location = 8) in vec3 a_position;
layout (location = 9) in vec4 a_color;
layout (location = 10) in float a_size;
layout (location = 11) in float a_rotation;



//uniforms
//layout(location = 0) uniform vec3 u_particlePosition;
//layout(location = 1) uniform vec4 u_particleColor;
//layout(location = 2) uniform float u_size;

// SceneUBO:
//THIS NEEDS TO MATCH SceneUBO.h!
const int LIGHT_COUNT = 50;
const int SHADOWMAP_COUNT = 3;
struct Light {
    vec4 position;
	vec4 direction;
    vec4 color;
	int type;

    float constant;
    float linear;
    float quadratic;
};
layout(std140, binding = 0) uniform Scene {
	mat4 u_ViewMatrix;
	mat4 u_ProjectMatrix;

	mat4 lightViewMatrices[4];
	mat4 lightOrthoProjMatrices[4];
	vec4 cascadeFarZ;				//we use a vec4 here casue glsl wants to pad each element of a float array up to a vec4.
	vec4 u_sunDirection;
	vec4 u_sunColor;

	vec4 u_Iambient;		//ambient illumination color
	int u_LightCount;

	Light u_Lights[LIGHT_COUNT];
};


//Varying
layout(location=1) out vec2 v_uv;
layout(location=20) out vec4 v_color;


mat4 rotationMatrix(vec3 axis, float angle)
{
    //from http://www.neilmendoza.com/glsl-rotation-about-an-arbitrary-axis/
    axis = normalize(axis);
    float s = sin(angle);
    float c = cos(angle);
    float oc = 1.0 - c;
    
    return mat4(oc * axis.x * axis.x + c,           oc * axis.x * axis.y - axis.z * s,  oc * axis.z * axis.x + axis.y * s,  0.0,
                oc * axis.x * axis.y + axis.z * s,  oc * axis.y * axis.y + c,           oc * axis.y * axis.z - axis.x * s,  0.0,
                oc * axis.z * axis.x - axis.y * s,  oc * axis.y * axis.z + axis.x * s,  oc * axis.z * axis.z + c,           0.0,
                0.0,                                0.0,                                0.0,                                1.0);
}


void main()
{
    mat4 mvp_Matrix = u_ProjectMatrix * u_ViewMatrix;// * u_ModelMatrix;

    vec3 cameraRight_worldspace = {u_ViewMatrix[0][0], u_ViewMatrix[1][0], u_ViewMatrix[2][0]};
    vec3 cameraUp_worldspace = {u_ViewMatrix[0][1], u_ViewMatrix[1][1], u_ViewMatrix[2][1]};

    //allow the billboard to rotate around their center.
    mat4 rotMatrix = rotationMatrix(vec3(0.0,0.0,-1.0), a_rotation);
    vec3 rotatedVertex = (rotMatrix * vec4(a_vertex, 1.0)).xyz;


    vec3 vertexPosition_worldspace = a_position
        + cameraRight_worldspace * rotatedVertex.x * a_size
        + cameraUp_worldspace * rotatedVertex.y * a_size;

    gl_Position = mvp_Matrix * vec4(vertexPosition_worldspace, 1.0);
    v_uv = a_UV;
    v_color = a_color;
}